/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static char rcsid[] =
"@(#) $Header: burst.c,v 1.2 91/11/12 14:03:30 floyd Exp $ (LBL)";
#endif

#include <stdio.h>
#include <sys/types.h>
#include <math.h>

#include "config.h"
#include "real.h"
#include "packet.h"
#include "tcp.h"
#include "at.h"

struct burstate {
        int size;
        float interval;
        int started;
        int timeout_no;
	int pktburst;
};

static struct at_options options[] = {
        { "interval", AT_FLOAT, offsetof(struct burstate, interval) },
        { "size", AT_INT, offsetof(struct burstate, size) },
	{ "pktburst", AT_INT, offsetof(struct burstate, pktburst) },
        { 0 },
};

static inline int
rnd_delay(f)
        float f;
{
        return (int)(-log(1.0 - random() / 2147483647.0) * f);
}

void
bur_init(ap)
        register struct application *ap;
{
        register struct burstate *stp;

        stp = (struct burstate *)malloc(sizeof *stp);
        bzero((char *)stp, sizeof(*stp));
        atl_init(ap->ap_nb, ap->ap_attr, options, (char *)stp);
        stp->interval *= 1e6;
        stp->timeout_no = 0;
        ap->ap_size = stp->size;
        ap->ap_type = TELNET;
        ap->ap_state = (void *)stp;
}

int
bur_send(ap)
        register struct application *ap;
{
        struct burstate *stp = (struct burstate *)ap->ap_state;

        if (!stp->started) {
                stp->started = 1;
                apptimer(ap->ap_proto->pr_cid, rnd_delay(stp->interval));
        }
        return 0;
}

void
bur_timeout(ap, id)
        struct application *ap;
        int id;
{

        struct burstate *stp = (struct burstate *)ap->ap_state;

        ++stp->timeout_no;
        (*ap->ap_proto->pr_async)(ap->ap_proto, 1);
        if (stp->timeout_no % stp->pktburst == 0)
          apptimer(ap->ap_proto->pr_cid, rnd_delay(stp->interval));
        else
          apptimer(ap->ap_proto->pr_cid, 0);

}
