/*
 * Copyright (c) 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: class.c,v 1.5 91/12/10 16:49:40 floyd Exp $ (LBL)";
#endif

#include <stdio.h>
#include "config.h"
#include "real.h"
#include "at.h"
#include "class.h"
#include "bind.h"

/*
 * list of default class nodes so we can
 * lookup and merge conversation memeber lists
 */
static struct class *cdef;

static void
enter_cdef(struct class *cl)
{
	cl->next = cdef;
	cdef = cl;
}

struct class *
classlookup(struct class *cl, char *name)
{
	name = intern(name);
	for (; cl != 0; cl = cl->next)
		if (cl->name == name)
			break;
	return cl;
}

struct idlist *
makeid(char *v, struct idlist *p)
{
	struct idlist *f = (struct idlist *)malloc(sizeof(*f));

	f->next = p;
	f->id = v;
	return f;
}

static int
inlist(char *id, struct idlist *p)
{
	for (; p != 0; p = p->next)
		if (p->id == id)
			return 1;
	return 0;
}

static struct idlist *
idmerge(struct idlist *old, struct idlist *new)
{
	for (; old != 0; old = old->next)
		if (!inlist(old->id, new))
			new = makeid(old->id, new);
	return new;
}

struct class *
classbuild(char *name, struct attr *attr, struct idlist *m, int lineno)
{
	struct class *cl, *template;
	
	cl = (struct class *)malloc(sizeof(*cl));
	template = classlookup(cdef, name);
	if (template != 0)
		cl->members = idmerge(template->members, m);
	else
		cl->members = m;
	cl->nb = nbclass(name);
	cl->name = name;
	cl->lineno = lineno;
	cl->attr = attr;

	return cl;
}

void
cl_default(struct class *cl)
{
	struct class *def;

	atl_default(cl->name, cl->attr);
	def = classlookup(cdef, cl->name);
	if (def != 0) {
		def->members = idmerge(cl->members, def->members);
		free((char *)cl);
	} else
		enter_cdef(cl);
}

struct class *
getclasslist(int eid, struct ifnet **p)
{
	struct r_edge *ep;
	struct r_node *np;

	for (np = nodelist; np != 0; np = np->rn_next)
		for (ep = np->rn_edgelist; ep != 0; ep = ep->re_next)
			if (ep->re_id == eid) {
				*p = ep->re_ifp;
				return ep->re_clist;
			}
	return 0;
}
