/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static char rcsid[] =
"@(#) $Header: ed2.c,v 1.13 93/06/14 12:47:40 mccanne Exp $ (LBL)";
#endif

#include <stdio.h>
#include <sys/types.h>

#include "real.h"
#include "packet.h"
#include "at.h"
#include "config.h"
#include "drop.h"
#include "pq.h"

extern double exp2();
extern double log2();
#ifndef sun
#define exp2(x) pow(2.0, (x))
#define log2(x) (log(x)/0.69314718056)
#endif

/*
static int earlydrop();
static void latedrop();
*/

/*
 * Early drop parameters.
 */
struct edp {
	/*
	 * User supplied.
	 */
	int option;
	int mean_pktsize;

	float th_min;
	float th_max;
	float minth_w;
	float th_inc;

	float r_inc;
	float r_max;

	float q_w;
	/*
	 * Computed as a function of user supplied paramters.
	 */
	float k;
	float ptc;		/* packet time constant */
};

/*
 * Early drop variables.
 */
struct edv {
	float v_th;
	float v_ave;
	float v_avelast;
	float v_slope;
	float v_r;
	float v_th_w;
	float avecnt;
	int count;
	struct dlist *drops;
};

struct edstate {
	/*
	 * Static state.
	 */
	int maxq;
	int avg_pktsize;
	dropfn drop;
	char *plot_fname;
	FILE *plotfile;
	struct edp edp;
	int doubleq;
	int dqthresh;

	/*
	 * Dynamic state.
	 */
	int idle;
	double idletime;
	struct pq q;
	struct edv edv;
};
#define STATE(qd) ((struct edstate *)((qd)->qd_state))

static struct at_options options[] = {
	{ "queue-size", AT_INT, offsetof(struct edstate, maxq) },
	{ "option", AT_INT, offsetof(struct edstate, edp.option) },
	{ "plot-file", AT_NSTRING, offsetof(struct edstate, plot_fname) },
	{ "thresh", AT_FLOAT, offsetof(struct edstate, edp.th_min) },
	{ "mean_pktsize", AT_INT, offsetof(struct edstate, edp.mean_pktsize) },
	{ "maxrange", AT_FLOAT, offsetof(struct edstate, edp.r_max) },
	{ "linterm", AT_FLOAT, offsetof(struct edstate, edp.th_inc) },
	{ "th_weight", AT_FLOAT, offsetof(struct edstate, edp.minth_w) },
	{ "q_weight", AT_FLOAT, offsetof(struct edstate, edp.q_w) },
	{ "ra_linterm", AT_FLOAT, offsetof(struct edstate, edp.r_inc) },
	{ "maxthresh", AT_FLOAT, offsetof(struct edstate, edp.th_max) },
	{ "dropmech", AT_DROP, offsetof(struct edstate, drop) },
	{ "doubleq", AT_BOOL, offsetof(struct edstate, doubleq) },
	{ "dqthresh", AT_INT, offsetof(struct edstate, dqthresh) },
	{ 0 },
};

static void
ed2_comp(stp, nbytes, m)
	struct edstate *stp;
	int nbytes;
	int m;
{
	float f, f_sl, f_old;

	++stp->edv.count; 

	stp->edv.v_th = (1 - stp->edv.v_th_w) * stp->edv.v_th +
		stp->edv.v_th_w * stp->edp.th_min;

	f = stp->edv.v_ave;
	f_sl = stp->edv.v_slope;
	while (--m >= 1) {
		f_old = f;
		f += f_sl;
		f *= (1.0 - stp->edp.q_w);
		f_sl *= (1.0 - 0.5 * stp->edp.q_w);
		f_sl += 0.5 * stp->edp.q_w * (f - f_old);
	}
	f_old = f;
	f += f_sl;
	f *= (1.0 - stp->edp.q_w);
	f += stp->edp.q_w * (float)nbytes;
	f_sl *= (1.0 - 0.5 * stp->edp.q_w);
	f_sl += 0.5 * stp->edp.q_w * (f - f_old);
	stp->edv.v_ave = f;
	stp->edv.v_slope = f_sl;
}

void
ed2_init(struct qdisc *qd, double ifspeed)
{
	struct edstate *stp = (struct edstate *)malloc(sizeof(*stp));

	bzero((char *)stp, sizeof(*stp));
	atl_init(qd->qd_nb, qd->qd_attr, options, (char *)stp);
	qd->qd_state = (void *)stp;

	/* Packet time constant -- units are pkts/usec */
	stp->edp.ptc = 1 / ifspeed * stp->edp.mean_pktsize;

	stp->edp.k = stp->edp.th_max - stp->edp.r_max;
	if (stp->edp.th_max - stp->edp.th_min != 0)
		stp->edp.k /= stp->edp.th_max - stp->edp.th_min;

	stp->edv.v_th = stp->edp.th_min;
	stp->edv.v_r = (float)stp->edp.r_max * stp->edp.mean_pktsize;
	stp->edv.v_ave = 0.0;
	stp->edv.v_slope = 0.0;
	stp->edv.drops = 0;
	stp->edv.count = 0;
	stp->edv.v_th_w = stp->edp.minth_w;
	stp->edv.avecnt = -1.0/(float)log2(1.0-(double)stp->edv.v_th_w);
	stp->edv.v_avelast = 0.0;

	pqinit(&stp->q, stp->plot_fname, qd);
	stp->plotfile = stp->q.file;
}

static void
ed2_plot(stp)
	struct edstate *stp;
{
	float t = realtime();

	fprintf(stp->plotfile, "a %g %g\n", t, stp->edv.v_ave);
	fprintf(stp->plotfile, "t %g %g\n", t, stp->edv.v_th);
	fprintf(stp->plotfile, "r %g %g\n", t, stp->edv.v_r);
}

struct packet *
ed2_send(struct qdisc *qd)
{
	struct edstate *stp = STATE(qd);
	struct packet *pkt;

	if (stp->doubleq)
		pkt = pqthdeq(&STATE(qd)->q, stp->dqthresh);
	else
		pkt = pqdeq(&STATE(qd)->q);

	if (pkt == 0) {
		stp->idle = 1;
		stp->idletime = realtime();
	} else
		stp->idle = 0;
	return pkt;
}

void
ed2_recv(struct qdisc *qd, struct packet *pkt)
{
	struct edstate *stp = STATE(qd);
	struct pq *q = &stp->q;
	int m;

	if (stp->idle) {
		stp->idle = 0;
		m = stp->edp.ptc * (realtime() - stp->idletime);
		/*
		 * Remove phantom packets from drop list. 
		 */
		if (m > 0)
		   (void)earlydrop(&stp->edv.drops, m*stp->edp.mean_pktsize);
	} else 
		m = 0;

	if (stp->edv.drops && earlydrop(&stp->edv.drops, pkt->size)) {
		/* 
		 * Drop packet that just arrived.
		 */
		qd_drop(qd, pkt);
		pkt = 0;
	}
	/*
	 * Update early drop state.
	 */
	ed2_comp(stp, q->size, m + 1);

	if (stp->edv.v_ave > stp->edv.v_th * stp->edp.mean_pktsize) {
		int range, nextdrop;
		float newth;

                if (stp->edv.v_ave >= stp->edp.th_max * stp->edp.mean_pktsize) {
                   (void)earlydrop(&stp->edv.drops,
                        stp->edp.r_max*stp->edp.mean_pktsize);
                }
		range = (int)q->len + 2*(stp->edv.count-1);
		if (range > (int)stp->edp.r_max * stp->edp.mean_pktsize) 
			range = (int)stp->edp.r_max * stp->edp.mean_pktsize;
		stp->edv.v_r = (float) range;
		nextdrop = (range < 1) ? 1 : random() % range;
		if (nextdrop < q->size)
			qd_drop(qd, pqrm_b(q, nextdrop));
		else if (nextdrop <= q->size + pkt->size) {
			qd_drop(qd, pkt);
			pkt = 0;
		} else
			latedrop(&stp->edv.drops, nextdrop - q->size);
	

		stp->edv.v_th += stp->edp.th_inc;
		if (stp->edv.v_th > stp->edp.th_max)
			stp->edv.v_th = stp->edp.th_max;
		if ((stp->edv.count > (int)stp->edp.th_max) &&
		    (stp->edv.count < 1000)) {
			stp->edv.avecnt = 0.9*stp->edv.avecnt 
				+ 0.1*(float)stp->edv.count;
			stp->edv.v_th_w = 1.0 - 
				1.0/(float) exp2(1.0/(double)stp->edv.avecnt);
		}
		stp->edv.count = 0;
	}
	/*
	 * If we didn't drop the packet above, send it to the interface,
	 * checking for absolute queue overflow.
	 */
	if (pkt != 0) {
		pqenq(q, pkt);
		if (q->size > stp->maxq * stp->edp.mean_pktsize) {
			qd_drop(qd, pqrm(q, (*stp->drop)(q->len)));
			stp->edv.v_r = stp->maxq * stp->edp.mean_pktsize;
		}
	}
	if (stp->plotfile)
		ed2_plot(stp);
}

int
ed2_len(struct qdisc *qd)
{
	return STATE(qd)->q.len;
}
