/*
 * Copyright (c) 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static char rcsid[] =
    "@(#) $Header: sasink.c,v 1.16 94/10/16 18:51:16 floyd Exp $ (LBL)";
#endif

#include <stdio.h>
#include <sys/types.h>

#include "real.h"
#include "config.h"
#include "at.h"
#include "packet.h"

struct sasinkstate {
	int size;
	int next;	/*  next packet expected  */
	int maxseen;	/* max packet number seen */
	char seen[MWS]; /* array of packets seen  */
};

static struct at_options options[] = {
	{ "ack-size", AT_INT, offsetof(struct sasinkstate, size) },
	{ 0 },
};

static void
transmit_ack(proto, stp, seqno, congested, rtt_sent)
	struct protocol *proto;
	struct sasinkstate *stp;
	int seqno;
	int congested;
	float rtt_sent;
{
	struct packet *pkt;
	int i, j, size = stp->size, delta = 6;

	PALLOC(pkt);
	pkt->type = ACK;
	pkt->seqno = seqno;
	pkt->cid = proto->pr_cid;
	pkt->dst = PROTODST(proto);
	pkt->congest = congested;
	pkt->rtt_sent = rtt_sent;
	/*
	 * Put hole info in ack.
	 */
	j = 0;
	i = stp->next;
	/* Look at "next" to "maxseen" so far: if not "seen" it is a hole. */
	for (; i < stp->maxseen && j < NSA; ++i)
		if (!stp->seen[i & MWM]) {
		/* beginning of hole */
		  if (stp->seen[(i-1) & MWM]) {
		      pkt->sa_start[j] = i;
		  }
		/* end of hole */
		  if (stp->seen[(i+1) & MWM]) {
		      pkt->sa_end[j++] = i;
		      size += delta;  
		      delta = 4;
		  }
		}
	pkt->size = size;
	/*
	 * Terminate list with -1.
	 */
	pkt->sa_start[j] = -1;
	if_output(proto->pr_ifp, pkt);
}

void
sasink_init(proto)
	struct protocol *proto;
{
	struct sasinkstate *stp = (struct sasinkstate *)malloc(sizeof(*stp));

	atl_init(proto->pr_nb, proto->pr_attr, options, (char *)stp);
	stp->next = 0;
	stp->maxseen = -1;
	bzero(stp->seen, sizeof(stp->seen));
	proto->pr_state = (void *)stp;
}

void
sasink_input(proto, pkt)
	struct protocol *proto;
	struct packet *pkt;
{
	struct sasinkstate *stp = proto->pr_state;
	int next = stp->next;
	int lastmax, i, congested;
	float rtt_sent;
	
	congested = pkt->congest;
	rtt_sent = pkt->rtt_sent;
	if ((int)pkt->seqno > stp->maxseen) {
		for (i=stp->maxseen+1; i<(int)pkt->seqno; ++i) 
		  stp->seen[i & MWM] = 0; 
		stp->maxseen = (int)pkt->seqno;
		stp->seen[stp->maxseen & MWM] = 1;
		stp->seen[(stp->maxseen + 1) & MWM] = 0;
		}
	if ((int)pkt->seqno >= next && (int)pkt->seqno <= stp->maxseen) {
		/*
		 * setting the sequence number.
		 * (next - 1) should be the last in sequence packet seen
		 */
		stp->seen[((int)pkt->seqno) & MWM] = 1;
		while (stp->seen[next & MWM] == 1) {next=next+1;}
		stp->next = next;
	}
	transmit_ack(proto, stp, next - 1, congested, rtt_sent);
	PFREE(pkt);
}

