/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static char rcsid[] =
"@(#) $Header: satcp.c,v 1.21 94/10/16 18:53:47 floyd Exp $ (LBL)";
#endif

#include <stdio.h>
#include <sys/types.h>

#include "config.h"
#include "real.h"
#include "packet.h"
#include "tcp.h"
#include "at.h"
#define PEND_PKTS 100

struct satcpstate {
	struct tcpstate cmn;
	u_int dupwnd;
	int recover;	/* highest packet sent before Fast Recovery */  
	int samax;	/* max hole sent */
	int timeout; 	/* Just sent a packet from a timeout */
	int px;		/* index to array pending */
	int pending[PEND_PKTS + 1];
			/* contains number of packet to be retransmitted,
			 * ends with -1 */
	int pipe;       /* estimate of number of packets in pipe 
			 *  during Fast Recovery */
	int fastrecov;  /* fast recovery state */
};

/*
 * Send as much data as the window will allow, not in Fast Recovery.  
 * In Fast Recovery, send as much data as the pipe model will allow.
 * In Fast Recovery, fill holes before sending new packets.
 * The link layer will do the buffering.
 * Assume that the receiver's advertised window is twice the sender's max
 * window.
 */
void
satcp_send(proto, stp)
	register struct protocol *proto;
	register struct satcpstate *stp;
{
	register int win, pkt, nextpkt;
	int sendnow = stp->cmn.sendnow;
	int findpkt = 1;

	if (stp->cmn.seqno >= stp->cmn.curseq)
		stp->cmn.curseq += (*proto->pr_app.ap_send)(&proto->pr_app);

	if (stp->fastrecov == 0) 
	   win = min(stp->cmn.cwnd, stp->cmn.wnd);
        else
	   win = 2*stp->cmn.wnd;
	findpkt = 1;
        while ( ( ( stp->fastrecov==0 &&
                  stp->cmn.seqno <= stp->cmn.last_ack + win &&
                  stp->cmn.seqno < stp->cmn.curseq) ||
                (stp->fastrecov==1 && stp->pipe < stp->cmn.cwnd
               && stp->cmn.seqno < stp->cmn.curseq) )
               && (findpkt == 1) ) {
           if (stp->cmn.overhead == 0 || sendnow == 1) {
             findpkt = 0;
             while ((pkt = stp->pending[stp->px]) >= 0 &&
                 pkt <= stp->cmn.last_ack + win && findpkt==0) {
                        if (pkt > stp->samax) {
                            stp->samax = pkt;
                            if (pkt > stp->cmn.last_ack) {
                                nextpkt = pkt;
                                findpkt = 1;
                            }
                        }
                        ++stp->px;
             }
	     /*
             if (findpkt==0 && stp->pending[stp->px]==-1)
	     */
	     if (findpkt==0 && stp->cmn.seqno >= stp->pending[stp->px])
                if (stp->cmn.seqno <= stp->cmn.last_ack + win) {
                    nextpkt = stp->cmn.seqno++;
                    findpkt = 1;
                 }
             if (findpkt == 1) {
                 tcp_output(proto, proto->pr_app.ap_type,
                        proto->pr_app.ap_size, nextpkt);
                 if (stp->cmn.seqno <= nextpkt)
                        stp->cmn.seqno = nextpkt + 1;
                 ++stp->pipe;
             }
	 } else {
                        /*
                         * Set a delayed send timeout.
                         */
                        if (stp->cmn.delsend_tid == 0) {
                         long d = ((random()/2147483647.0)*stp->cmn.overhead);
                         stp->cmn.delsend_tid = prototimer(proto->pr_cid, d);
                        }
                        return;
                }
        }
        if (sendnow == 1) stp->cmn.sendnow = 0;
}
	
void
satcp_init(proto)
	register struct protocol *proto;
{
	register struct satcpstate *stp;

	stp = (struct satcpstate *)malloc(sizeof(*stp));
	tcp_cmninit(proto, &stp->cmn);
	stp->timeout = 0;
	stp->dupwnd = 0;
	stp->pending[stp->px = 0] = -1;
	stp->fastrecov = 0;
	proto->pr_state = (void *)stp;
}

void
satcp_input(proto, pkt)
	struct protocol *proto;
	struct packet *pkt;
{
	struct satcpstate *stp = proto->pr_state;
	int i, j, k;

	if (pkt->type != ACK)
		error("tcp received non-ack (probably talking to non-sink)");

	for (i = j = 0; pkt->sa_start[i] >= 0; ++i) {
		if (i >= NSA)
			/* XXX */
			fatal("satcp_input");
		for (k = pkt->sa_start[i]; k <= pkt->sa_end[i]; ++k) {
			if (k > stp->samax && j <= PEND_PKTS - 1) {
				stp->pending[j++] = k;
			}
		}
	}
	if (j <= PEND_PKTS ) stp->pending[j] = -1;
	stp->px = 0;

	if (pkt->congest==1)
		tcp_quench(&stp->cmn, 1);
	if ((int)pkt->seqno >= stp->cmn.seqno) {
		stp->cmn.seqno = (int)pkt->seqno + 1;
	}
	if  (stp->fastrecov == 0) {
        	if ( (int)pkt->seqno > stp->cmn.last_ack) {
        		tcp_newack(proto, &stp->cmn, pkt);
                       	tcp_increase(&stp->cmn);
        		stp->timeout = 0;
        	}
        	else if (stp->timeout == 0) {
        	  if (++stp->cmn.dupacks == NUMDUPACKS) {
        		/*
        		 * Assume we dropped a packet.  
			 * Retransmit last ack + 1
        		 * and begin Fast Recovery.
			 * Reset timer at start of Fast Recovery.
        		 */
        		stp->recover = stp->cmn.seqno - 1;
			stp->pipe = stp->cmn.cwnd - NUMDUPACKS + 1;
        		tcp_retx(proto, &stp->cmn, 1);
			stp->fastrecov = 1;
			stp->samax = stp->cmn.last_ack + 1;
			tcp_output(proto, proto->pr_app.ap_type,
				proto->pr_app.ap_size, stp->cmn.last_ack + 1);
        	  }
        	}
        	if (stp->cmn.dupacks == 0) 
			satcp_send(proto, stp);
	}
	else {
		--stp->pipe;
                if ( (int)pkt->seqno >= stp->recover ) 
                   {
			stp->fastrecov = 0;
                        tcp_newack(proto, &stp->cmn, pkt);
                        stp->timeout = 0;
                }
                else if ( (int)pkt->seqno > stp->cmn.highest_ack) {
                          stp->cmn.highest_ack = pkt->seqno;
                          stp->cmn.tcp.backoff = 0;
                          tcp_newtimer(proto, &stp->cmn, pkt);
			  /* To eliminate burstiness: 
			   * Set window to W/2 - K, not W/2,
			   * for K the number of dropped packets,
			   * and W the old window.
			  if (stp->cmn.dupacks > 0) {
			     stp->cwnd = stp->cmn.dupacks - stp->cwnd;
			     stp->cmn.dupacks = 0;
			  }
			  */
                        }
                else if (stp->timeout == 0) {
			if (stp->cmn.dupacks > 0) ++stp->cmn.dupacks;
                }
                satcp_send(proto, stp);
	}
	if ((int)pkt->seqno > stp->cmn.last_ack)
		stp->cmn.last_ack = (int)pkt->seqno;

	PFREE(pkt);
	if (stp->cmn.plotfile != 0)
		tcp_plot(&stp->cmn);
}

int
satcp_timeout(proto, id, arg)
	struct protocol *proto;
	int id;
	void *arg;
{
	struct satcpstate *stp = proto->pr_state;

	/* retransmit timer */
	if (stp->cmn.timer_id == id) {
		tcp_decrease(&stp->cmn, 0);
		tcp_retx(proto, &stp->cmn, 0);
		/* XXX */
		stp->samax = -1;
		stp->pending[stp->px = 0] = -1;
		stp->cmn.dupacks = 0;
		stp->timeout = 1;
		stp->recover = 0;
		stp->fastrecov = 0;
		stp->cmn.dupacks = 0;
                stp->cmn.sendnow = 1;
		satcp_send(proto, stp);
		return 1;
	/* delayed-send timer, for random overhead to avoid phase effects */
	} else if (stp->cmn.delsend_tid == id) {
                stp->cmn.delsend_tid = 0;
                stp->cmn.sendnow = 1;
                (*proto->pr_send)(proto, stp);
                return 1;
        }
	return 0;
}
