/*
 * Copyright (c) 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
"@(#) $Header: ur.c,v 1.6 92/11/30 10:50:59 floyd Exp $ (LBL)";
#endif

#include <stdio.h>
#include <sys/types.h>

#include "config.h"
#include "real.h"
#include "packet.h"
#include "tcp.h"
#include "at.h"

struct urstate {
	float maxpkts;
	int delay;
	int curseq;
	int seqno;
};

static struct at_options options[] = {
	{ "stop-at-packet", AT_FLOAT, offsetof(struct urstate, maxpkts) },
	{ "start-at", AT_USEC, offsetof(struct urstate, delay) },
	{ 0 },
};

void
ur_output(proto, type, size, seqno, maxrate, avgrate, pri)
	register struct protocol *proto;
	register int type;
	register int size;
	register int seqno;
	register int maxrate;
	register int avgrate;
	register int pri;
{
	register struct packet *pkt;
	struct urstate *stp = proto->pr_state;

	if (stp->maxpkts > 0.0 && (float) seqno > stp->maxpkts*1000.0)
		return;

	PALLOC(pkt);
	pkt->type = type;
	pkt->size = size;
	pkt->pri = pri;
	pkt->seqno = seqno;
	pkt->maxrate = (double) maxrate;
	pkt->avgrate = (double) avgrate;
	pkt->cid = proto->pr_cid;
	pkt->dst = PROTODST(proto);
	if_output(proto->pr_ifp, pkt);
}

/*
 * Try to send as much data as the window will allow.  The link layer will 
 * do the buffering; we ask the application layer for the size of the packets.
 */
void
ur_send(register struct protocol *proto, void *arg)
{
	register struct urstate *stp = (struct urstate *)arg;

        if (stp->seqno >= stp->curseq) {
                stp->curseq += (*proto->pr_app.ap_send)(&proto->pr_app);
                if (stp->seqno > stp->curseq) stp->seqno=stp->curseq;
        }

	while (stp->seqno < stp->curseq)
		ur_output(proto, proto->pr_app.ap_type, 
			  proto->pr_app.ap_size, stp->seqno++, 
			  (int) proto->pr_app.maxrate,
			  (int) proto->pr_app.avgrate,
			  (int) proto->pr_app.pri);
}

void
ur_async(struct protocol *proto, int data)
{
	register struct urstate *stp = proto->pr_state;

	stp->curseq += data;
	(*proto->pr_send)(proto, (void *)stp);
}
void
ur_init(register struct protocol *proto)
{
	register struct urstate *stp;

	stp = (struct urstate *)malloc(sizeof *stp);
	bzero((char *)stp, sizeof(*stp));

	proto->pr_state = (void *)stp;
	proto->pr_app.ap_proto = proto;

	(proto->pr_app.ap_init)(&proto->pr_app);
	atl_init(proto->pr_nb, proto->pr_attr, options, (char *)stp);

	stp->seqno = 0;
}

int
ur_timeout(struct protocol *proto, int id)
{
	return 0;
}

static int
ur_delay(struct protocol *proto, int id)
{
	struct urstate *stp = proto->pr_state;

	proto->pr_timeout = ur_timeout;
	(*proto->pr_send)(proto, proto->pr_state);
	return 0;
}

int
ur_start(struct protocol *proto)
{
	struct urstate *stp = proto->pr_state;

	if (stp->delay > 0) {
		proto->pr_timeout = ur_delay;
		(void)prototimer(proto->pr_cid, stp->delay);
		stp->delay = 0;
	} else
		(*proto->pr_send)(proto, stp);
}

void
ur_input(struct protocol *proto, struct packet *pkt)
{
	if (pkt->type != ACK)
		error("ur received non-ack (routing problem?)");

	PFREE(pkt);
}



