/*
 * Copyright (c) 1990 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static char rcsid[] =
"@(#) $Header: vid.c,v 1.4 91/10/07 15:17:52 floyd Exp $ (LBL)";
#endif

#include <stdio.h>
#include <sys/types.h>
#include <math.h>

#include "config.h"
#include "real.h"
#include "packet.h"
#include "tcp.h"
#include "at.h"

struct vidstate {
	int size;
	int interval;
	int min_pkts;
	int max_pkts;
	int tot_pkts;
	int started;
	int frame;
	int count1;
	int left;
};

static struct at_options options[] = {
	{ "interval", AT_USEC, offsetof(struct vidstate, interval) },
	{ "size", AT_INT, offsetof(struct vidstate, size) },
	{ "min_pkts", AT_INT, offsetof(struct vidstate, min_pkts) },
	{ "max_pkts", AT_INT, offsetof(struct vidstate, max_pkts) },
	{ "tot_pkts", AT_INT, offsetof(struct vidstate, tot_pkts) },
	{ 0 },
	{ 0 },
	{ 0 },
	{ 0 },
};

void
vid_init(ap)
	register struct application *ap;
{
	register struct vidstate *stp;

	stp = (struct vidstate *)malloc(sizeof *stp);
	bzero((char *)stp, sizeof(*stp));
	atl_init(ap->ap_nb, ap->ap_attr, options, (char *)stp);
	ap->ap_size = stp->size;
	ap->ap_type = VIDEO;
	ap->ap_state = (void *)stp;
	stp->count1 = stp->min_pkts + random()%(stp->max_pkts - stp->min_pkts);
	stp->left = stp->tot_pkts - stp->count1;
	stp->frame = 1;
}

int
vid_send(ap)
	register struct application *ap;
{
	struct vidstate *stp = (struct vidstate *)ap->ap_state;

	if (!stp->started) {
		stp->started = 1;
		apptimer(ap->ap_proto->pr_cid, stp->interval);
	}
	return 0;
}

void
vid_timeout(ap, id)
	struct application *ap;
	int id;
{
	struct vidstate *stp = (struct vidstate *)ap->ap_state;
	if (stp->frame == 0) {
	  stp->frame = 1;
          stp->count1 = stp->min_pkts + random()%(stp->max_pkts - stp->min_pkts);
          stp->left = stp->tot_pkts - stp->count1;
	}
	--stp->count1;
	(*ap->ap_proto->pr_async)(ap->ap_proto, 1);
	if (stp->count1 > 0)
	  apptimer(ap->ap_proto->pr_cid, stp->interval);
	else {
	   stp->frame = 0;
	   apptimer(ap->ap_proto->pr_cid, stp->left * stp->interval);
	}
}
