/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: data-source.cc,v 1.7 95/11/05 15:53:04 mccanne Exp $ (LBL)";
#endif

#include <stdlib.h>
#include "data-source.h"
#include "random.h"

DataSource::DataSource(DataHandler* dh) : dh_(dh)
{
}

int DataSource::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "start") == 0) {
			start();
			return (TCL_OK);
		}
#ifdef notyet
		if (strcmp(argv[1], "stop") == 0) {
			stop();
			return (TCL_OK);
		}
#endif
	} else if (argc == 3) {
		if (strcmp(argv[1], "produce") == 0) {
			dh_->advance(atoi(argv[2]));
			return (TCL_OK);
		}
	}
	return (NsObject::command(argc, argv));
}

FtpSource::FtpSource(DataHandler* dh) : DataSource(dh)
{
}

void FtpSource::start()
{
	dh_->advance(0x10000000);
}

TelnetSource::TelnetSource(DataHandler* dh) : DataSource(dh)
{
	link_real("ns_telnet", "interval", &interval_, 0);
}

void TelnetSource::start()
{
	timeout();
}

void TelnetSource::timeout()
{
	dh_->advance(1);
	sched();
}

void TelnetSource::handle(Event*)
{
	timeout();
}

extern "C" double tcplib_telnet_interarrival();

void TelnetSource::sched()
{
	double del;

	if (interval_ == 0.)
		/* use tcplib */
		del = tcplib_telnet_interarrival();
	else
		del = Random::exponential() * interval_;

	Scheduler::instance().schedule(this, &timer_, del);
}
