/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: drop-tail.cc,v 1.8 95/11/05 15:52:54 mccanne Exp $ (LBL)";
#endif

#include <string.h>
#include "link.h"

/*
 * A bounded, drop-tail queue
 */
class DropTailLink : public Link {
 protected:
	void enque(Packet*);
	Packet* deque();
	Queue q_;
};

static class DropTailMatcher : public Matcher {
public:
	DropTailMatcher() : Matcher("link") {}
	TclObject* match(const char* id) {
		if (strcasecmp(id, "drop-tail") == 0)
			return (new DropTailLink);
		return (0);
	}
} matcher_drop_tail;

/*
 * drop-tail
 */
void DropTailLink::enque(Packet* p)
{
	q_.enque(p);
	log_packet_arrival(p);
	if (qnp_ >= qlim_) {
		q_.remove(p);
		log_packet_drop(p);
		Packet::free(p);
	}
}

Packet* DropTailLink::deque()
{
	Packet* p = q_.deque();
	if (p != 0)
		log_packet_departure(p);
	return (p);
}
