/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: loss-monitor.cc,v 1.4 95/11/22 15:09:24 mccanne Exp $ (LBL)";
#endif

#include "agent.h"
#include "node.h"
#include "Tcl.h"
#include "packet.h"

class LossMonitor : public Agent {
 public:
	LossMonitor();
	int command(int argc, const char*const* argv);
	void recv(Packet* pkt);
protected:
	char* proc_;
	int loss_;
	int expected_;
	double last_packet_time_;
};

static class LossMonitorMatcher : public Matcher {
public:
	LossMonitorMatcher() : Matcher("agent") {}
	TclObject* match(const char* id) {
		if (strcmp(id, "loss-monitor") == 0)
			return (new LossMonitor());
		return (0);
	}
} matcher_cbr;

LossMonitor::LossMonitor() : Agent(-1), proc_(0), expected_(-1)
{
	loss_ = 0;
	last_packet_time_ = 0.;
	link_int(0, "loss", &loss_, 1);
	link_real(0, "last-packet-time", &last_packet_time_, 1);
}

void LossMonitor::recv(Packet* pkt)
{
	/*
	 * Check for lost packets
	 */
	last_packet_time_ = Scheduler::instance().clock();
	if (expected_ >= 0) {
		int loss = pkt->seqno_ - expected_;
		if (loss > 0) {
			loss_ += loss;
			if (proc_ != 0)
				Tcl::instance().eval(proc_);
		}
	}
	expected_ = pkt->seqno_ + 1;
	Packet::free(pkt);
}

/*
 * $proc interval $interval
 * $proc size $size
 */
int LossMonitor::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "clear") == 0) {
			expected_ = -1;
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "proc") == 0) {
			const char* proc = argv[2];
			int n = strlen(proc) + 1;
			delete proc_;
			proc_ = new char[n];
			strcpy(proc_, proc);
			return (TCL_OK);
		}
	}
	return (Agent::command(argc, argv));
}
